package app;

import java.util.ArrayList;
import java.util.PriorityQueue;
import java.util.Queue;

import enums.MessageType;
import gui.MainWindow;
import interfaces.MessageGenerator;
import interfaces.MessageQueue;

public class MessageQueueImpl implements MessageQueue {

	// Variables
	private Queue<String> messagequeue;
	private ArrayList<MessageGenerator> generators;
	private MainWindow mainwindow;

	// Construktor
	public MessageQueueImpl(MainWindow mainwindow) {
		// Inject reference to the GUI (to send messages to the log box).
		this.mainwindow = mainwindow;

		// Create queue to store the messages in.
		messagequeue = new PriorityQueue<>();

		// Create collection to store created threads in (for pause/resume).
		generators = new ArrayList<>();
	}

	///////////////////

	@Override
	public void createProducers(int numberOfProducers) {
		// Create a set amount of producers, start them, and store them.
		for (int i = 1; i <= numberOfProducers; i++) {
			MessageGenerator messagegenerator = new MessageGeneratorImpl(this, "msgGen" + i);
			messagegenerator.start();
			generators.add(messagegenerator);
		}
	}

	@Override
	public void startAllProducers() {
		// Resume all stored threads.
		for (MessageGenerator generator : generators) {
			generator.start();
		}
	}

	@Override
	public void pauseAllProducers() {
		// Do I really need to explain this?
		for (MessageGenerator generator : generators) {
			generator.pause();
		}
	}

	@Override
	public void addToQueue(String message, MessageType type) {
		// Assemble output string.
		String messageToAdd = "<" + type.toString() + "> " + message;

		// Add given message to the tail of the queue.
		messagequeue.offer(messageToAdd);

		// TODO diesen shit sinnvoll queuen
		// Write the head message from the queue to the window.
		mainwindow.writeToLog(messagequeue.poll());
	}
}
