package gui;

import java.awt.EventQueue;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.ScrollPaneConstants;
import javax.swing.UIManager;
import javax.swing.border.EtchedBorder;

import app.MessageQueueImpl;
import enums.MessageType;
import interfaces.MessageQueue;

public class MainWindow {

	private JFrame frmNachrichtenticker;
	private JTextField textFieldNewMessage;
	private JTextArea textareaMessageLog;
	private MessageQueue messagequeue;
	private JScrollPane scrollPane;

	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		try {
			UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
		} catch (Throwable e) {
			e.printStackTrace();
		}
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					MainWindow window = new MainWindow();
					window.frmNachrichtenticker.setVisible(true);

					int numberOfThreads = 1;
					if (args.length > 0) {
						numberOfThreads = Integer.parseInt(args[0]);
					}

					window.initMessageThreads(window, numberOfThreads);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});

	}

	private void initMessageThreads(MainWindow mainwindow, int numberOfProducers) {
		messagequeue = new MessageQueueImpl(mainwindow);
		messagequeue.createProducers(numberOfProducers);
	}

	public void writeToLog(String message) {
		textareaMessageLog.append(message + "\n");
		textareaMessageLog.setCaretPosition(textareaMessageLog.getDocument().getLength());
	}

	/**
	 * Create the application.
	 */
	public MainWindow() {
		initialize();
	}

	/**
	 * Initialize the contents of the frame.
	 */
	@SuppressWarnings("unchecked")
	private void initialize() {
		frmNachrichtenticker = new JFrame();
		frmNachrichtenticker.setTitle("Nachrichten-Ticker");
		frmNachrichtenticker.setBounds(100, 100, 704, 493);
		frmNachrichtenticker.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frmNachrichtenticker.getContentPane().setLayout(null);

		scrollPane = new JScrollPane();
		scrollPane.setBounds(10, 11, 352, 432);
		frmNachrichtenticker.getContentPane().add(scrollPane);
		scrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
		scrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);

		textareaMessageLog = new JTextArea();
		scrollPane.setViewportView(textareaMessageLog);
		textareaMessageLog.setEditable(false);
		textareaMessageLog.setLineWrap(true);

		JPanel panel = new JPanel();
		panel.setBorder(new EtchedBorder(EtchedBorder.LOWERED, null, null));
		panel.setBounds(372, 11, 306, 107);
		frmNachrichtenticker.getContentPane().add(panel);
		panel.setLayout(null);

		JLabel lblNeueNachricht = new JLabel("New Message:");
		lblNeueNachricht.setBounds(10, 11, 84, 14);
		panel.add(lblNeueNachricht);

		textFieldNewMessage = new JTextField();
		textFieldNewMessage.setBounds(10, 36, 286, 20);
		panel.add(textFieldNewMessage);
		textFieldNewMessage.setColumns(10);

		@SuppressWarnings("rawtypes")
		JComboBox comboBoxMessageType = new JComboBox();

		DefaultComboBoxModel<MessageType> comboboxmodel = new DefaultComboBoxModel<>();
		for (MessageType nt : MessageType.values()) {
			comboboxmodel.addElement(nt);
		}
		comboBoxMessageType.setModel(comboboxmodel);
		comboBoxMessageType.setBounds(10, 67, 182, 20);

		panel.add(comboBoxMessageType);

		JButton btnNewMessage = new JButton("Send");
		btnNewMessage.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				messagequeue.addToQueue(textFieldNewMessage.getText(),
						(MessageType) comboBoxMessageType.getSelectedItem());
			}
		});
		btnNewMessage.setBounds(202, 67, 94, 23);
		panel.add(btnNewMessage);

		JButton btnStartThreads = new JButton("Start Threads");
		btnStartThreads.setEnabled(false);
		btnStartThreads.setBounds(372, 129, 306, 23);
		frmNachrichtenticker.getContentPane().add(btnStartThreads);

		JButton btnPauseThreads = new JButton("Pause Threads");
		btnPauseThreads.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				btnStartThreads.setEnabled(true);
				btnPauseThreads.setEnabled(false);

				messagequeue.pauseAllProducers();
			}
		});
		btnPauseThreads.setBounds(372, 163, 306, 23);
		frmNachrichtenticker.getContentPane().add(btnPauseThreads);
		btnStartThreads.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				btnStartThreads.setEnabled(false);
				btnPauseThreads.setEnabled(true);

				messagequeue.startAllProducers();
			}
		});
	}
}
